#include <netinet/in.h> 
#include <sys/socket.h>
#include <ace/Log_Msg.h>
#include <ace/Get_Opt.h>
#include "DBBalancer.hh"
#include "DBBalancerDaemon.hh"

#ifdef XML_CONFIG
#include "config/DBBalancerXMLConfig.hh"
#else
#include "config/DBBalancerTxtConfig.hh"
#endif


DBBalancerDaemon::DBBalancerDaemon(DBBalancer* db, int port) {

  //cout << "Construyendo DBBalancerDaemon." << endl;
  MAJ_VERSION = db->getMajVersion();
  MIN_VERSION = db->getMinVersion();
  PATCH_LEVEL = db->getPatchLevel();
  STATUS = db->getStatus();
  PORT = port;
  _db = db;
  _it = 0;
  
}

DBBalancerDaemon::~DBBalancerDaemon() {

  ACE_DEBUG((LM_DEBUG,"Destruyendo DBBalancerDaemon.\n"));

}


void DBBalancerDaemon::hello() {

  ACE_DEBUG((LM_INFO,"\nDBBalancerDaemon version %d.%d.%d %s: Port %d.\n",MAJ_VERSION,MIN_VERSION,PATCH_LEVEL, STATUS.c_str(), PORT));
  ACE_DEBUG((LM_INFO,"------------------------------------------------------\n"));
  ACE_DEBUG((LM_INFO,"(c) 2000,2001. Daniel Varela\n"));
  ACE_DEBUG((LM_INFO,"------------------------------------------------------\n\n"));


}

void DBBalancerDaemon::run () {

  int          sd, sd_current;
  unsigned int addrlen;
  struct       sockaddr_in sin;
  struct       sockaddr_in pin;

  // Get an internet domain socket
  if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    ACE_ERROR((LM_ERROR,"(%t) Error creating socket: %m"));
    exit(1);
  }

  // Complete the socket structure
  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(PORT);
  
  // Bind the socket to the port number
  if (bind(sd, (struct sockaddr *) &sin, sizeof(sin)) == -1) {
    ACE_ERROR((LM_CRITICAL,"(%t) Error binding to socket: %m\n"));
    //throw this;
    ACE_OS::exit(1);
  }
  
  // Show that we are willing to listen
  if (listen(sd, 5) == -1) {
    ACE_ERROR((LM_CRITICAL,"(%t) Error when starting to listen: %m\n"));
    ACE_OS::exit(1);
  }

  // Main request accepting loop.
  while (1) { 
    
    ACE_DEBUG((LM_DEBUG,"(%t) Waiting for a connection at port: %d\n", PORT));
    
    // Wait for a client to talk to us
    if ((sd_current = accept(sd, (struct sockaddr *)  &pin, &addrlen)) == -1) {
      ACE_ERROR((LM_ERROR,"(%t) Error accepting client: %m"));
      break;
    }

    ACE_DEBUG((LM_DEBUG,"(%t) Connection received.\n"));

    _db->processConnection(sd_current);

    _it++;

  }

  close(sd);                    
}


void usage() {

  ACE_DEBUG((LM_NOTICE,"Usage: DBBalancerDaemon [-r|-w] [-d] [-f config_file]\n"));

}


int main(int argc, char *argv[]) {

  char c;
  bool ok = true;

  // Default config values
  string config_file="config/DBBalancerConfig.conf";
  ACE_LOG_MSG->disable_debug_messages();    

  u_long priority_mask = ACE_LOG_MSG->priority_mask (ACE_Log_Msg::PROCESS);
  ACE_CLR_BITS (priority_mask, LM_DEBUG);
  ACE_LOG_MSG->priority_mask (priority_mask, ACE_Log_Msg::PROCESS);

  unsigned int mode = DBBalancer::READER_PROCESS;


  ACE_Get_Opt get_opt(argc,argv,"drwf:",1,1);

  while ((c=get_opt())!=EOF) 

    switch (c) {

    case 'd':

      ACE_LOG_MSG->enable_debug_messages();      
      ACE_DEBUG((LM_DEBUG,"Enabling DEBUG mode.\n",c));      
      break;

    case 'r':
      // This is a reader process.
      ACE_DEBUG((LM_DEBUG,"Reader Process: %c\n",c));      
      mode = DBBalancer::READER_PROCESS;

      break;

    case 'w':

      // This is a writer process;
      ACE_DEBUG((LM_DEBUG,"Writer Process: %c\n",c));
      mode = DBBalancer::WRITER_PROCESS;
      break;

    case 'f':

      // The path to the config file;
      config_file = get_opt.optarg;
      ACE_DEBUG((LM_NOTICE,"Config File: %s\n",config_file.c_str()));
      break;

    default:

      ok = false;

    }

  if (!ok) {
    usage();
    ACE_OS::exit(-1);
  }


#ifdef XML_CONFIG
  DBBalancerConfig *config = new DBBalancerXMLConfig(config_file);
#else
  DBBalancerConfig *config = new DBBalancerTxtConfig(config_file);
#endif

  DBBalancer *db = new DBBalancer(config,mode);
  DBBalancerDaemon *d = new DBBalancerDaemon(db,(mode==DBBalancer::READER_PROCESS)?config->getDaemonReaderPort():config->getDaemonWriterPort());

  d->hello();
  d->run();

  delete d;
  delete db;
  delete config;

  

}

